module DataStructures.Tree where

data Tree a = Tree {
	content :: a,
	children :: Forest a
	}
	deriving (Eq)

type Forest a = [Tree a]


instance Show a => Show (Tree a) where
	show = unlines . treeToStrings where
		
		forestToStrings :: Show a => Forest a -> [String]
		forestToStrings forest = concat $ mapLast (norm_tree_prefix) (last_tree_prefix) trees_strings where
			trees_strings = map treeToStrings forest 
			norm_tree_prefix = mapFirst ((++) "+-- ") ((++) "|   ")
			last_tree_prefix = mapFirst ((++) "`-- ") ((++) "    ")
		
		treeToStrings :: Show a => Tree a -> [String]
		treeToStrings tree = lines (show (content tree)) ++ forestToStrings (children tree)


mapFirst :: (a -> b) -> (a -> b) -> [a] -> [b]
mapFirst funF funN list = case list of
	[]     -> []
	x:xs   -> (funF x) : (map funN xs)

mapLast :: (a -> b) -> (a -> b) -> [a] -> [b]
mapLast funN funL list = (reverse . mapFirst funL funN . reverse) list




tFlatten :: Tree a -> [a]
tFlatten tree = content tree : fFlatten (children tree)

fFlatten :: Forest a -> [a]
fFlatten forest = concat (map tFlatten forest)

tMap :: (a -> b) -> Tree a -> Tree b
tMap fun tree = Tree {
	content = fun (content tree),
	children = fMap fun (children tree)
	}

fMap :: (a -> b) -> Forest a -> Forest b
fMap fun forest = map (tMap fun) forest

toForest :: (a -> a -> Bool) -> [a] -> Forest a
toForest isChild []   = []
toForest isChild list = tree : toForest isChild remaining where
	first = head list
	tree = Tree {
		content = first,
		children = toForest isChild subList
	}
	
	(subList, remaining) = span (isChild first) (tail list)

