module SyntaxEngine.Parsing where

import InternalRepresentation.Tokens

import SyntaxEngine.Pos
import SyntaxEngine.Result

import DataStructures.Tree


data ParseNode token ptype
	= Node ptype
	| Leaf token
	deriving (Eq, Show)


type Parser token ptype = [token] -> Result (Tree (ParseNode token ptype), [token]) String


type ParserWP token ptype = [WithPos token] ->  Result (Tree (WithPos (ParseNode token ptype)), [WithPos token]) ErrorMsg


--makeParserWP :: Parser -> ParserWP


{-
data ParseDef 
	= Sequence [ParseDef]
	| Alternatives [ParseDef]
	
	| ListOf ParseDef IsToken

	| Take IsToken
	| Skip IsToken
	| Parse PType

	| Reorder [Int]




type IsToken = Token -> Bool


	


list_of :: ParseDef -> ParseDef
list_of pDef = Sequence [pDef]--, Maybe (list_of pDef)]


data PType
	= Value
	| Symbol
	| Member
	| FunCall
	| FunDef
	| Definition
	deriving (Eq, Show)


parse :: PType -> ParseDef
parse Definition = Sequence [
	symbol,
	keyword "=",
	Alternatives [
		--expr, 
		funDef]
	]


parse FunDef = Sequence [
	keyword "function",
	list_of symbol,
	keyword "->",
	list_of symbol --expr
	]

{-
parse Expr = many+ (symbol <or> value <or> subExpr) <recombined>

parse SubExpr = Sequence [
	delimiter "(",
	expr,
	delimiter ")"
	]
-}


funDef :: ParseDef
funDef = Parse FunDef

--expr :: ParseDef
--expr = parse Expr



keyword :: String -> ParseDef
keyword k = Take isKeyword where
	isKeyword token = case token of
		TKeyword k' | k == k' -> True
		otherwise             -> False

symbol :: ParseDef
symbol = Take isSymbol where
	isSymbol token = case token of
		TSymbol _ -> True
		otherwise -> False 

delimiter :: String -> ParseDef
delimiter d = Take isDelimiter where
	isDelimiter token = case token of
		TDelimiter d' | d == d' -> True
		otherwise               -> False

















{-
In:
	tokenize "a - b * c - d"

Out:
	Tree FunCall
		Tree Symbol -
		Tree FunCall
			Tree Symbol -
			Tree Symbol a
			Tree FunCall
				Tree Symbol *
				Tree Symbol b
				Tree Symbol c
		Tree Symbol d


In:
	tokenize "this.that.member"
Out:
	Tree Atom
		Tree Atom
			Tree Symbol this
			Tree Symbol that
		Tree Symbol member

-}
-}