module SyntaxEngine.Pos where

data Pos = Pos {
	line :: Int,
	char :: Int
	}
	deriving (Eq)

data WithPos a = WithPos {
	inside :: a,
	start :: Pos,
	end :: Pos
	}
	deriving (Eq)

type ErrorMsg = WithPos String


instance Show Pos where
	show (Pos i j) = "(" ++ show i ++ "," ++ show j ++ ")"

instance Show a => Show (WithPos a) where
	show (WithPos i s e) = show s ++ "-" ++ show e ++ ": " ++ show i


mapInside :: (a -> b) -> [WithPos a] -> [WithPos b]
mapInside fun list = map funInside list where
	funInside wp = wp {inside = fun (inside wp)}



withSamePos :: a -> WithPos b -> WithPos a
withSamePos x wp = wp {
	inside = x
	}

withPosRange :: a -> (WithPos b, WithPos c) -> WithPos a
withPosRange x (s,e) = WithPos {
	inside = x,
	start = start s,
	end = end e
	}


shift :: Pos -> Int -> Pos
shift pos nbChars = pos {
	char = char pos + nbChars
	}



{-
consume taken (pos, str) = (pos', str') where
	str' = drop (length taken) str
	pos' = shift pos (length str + 1)
-}

